@@ -1,9 +1,11 @@
-//go:build go1.7
-// +build go1.7
+//go:build !go1.7
+// +build !go1.7
 
 package aws
 
-import "context"
+import (
+	"github.com/aws/aws-sdk-go/internal/context"
+)
 
 // BackgroundContext returns a context that will never be canceled, has no
 // values, and no deadline. This context is used by the SDK to provide
@@ -17,5 +19,5 @@ import "context"
 //
 // See https://golang.org/pkg/context for more information on Contexts.
 func BackgroundContext() Context {
-	return context.Background()
+	return context.BackgroundCtx
 }
